# SMOOTH_STRATEGY = 'inverse'
# SMOOTH_NOISE = 1.0
#
# SEGMENT_EPS = 0.15
# SEGMENT_MIN_SAMPLES = 80
# SEGMENT_MIN_TIME = 60
#
# # SIMPLIFY_MAX_TIME = 5
# # SIMPLIFY_MAX_DISTANCE = 0.01
# SIMPLIFY_DISTANCE_THRESHOLD = 30
# SIMPLIFY_EPS = 0.0001
#
# TM_REMOVE_STOPS = False
# TM_DT_THRESHOLD = 10
#
# LOCATIONS_LIMIT = 5
# LOCATION_MAX_DISTANCE = 20 # Distance in meters
#
# PREPROCESS_MAX_ACC = 100000.0
